﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Globalization;
using System.IO;
using System.Text;
using Microsoft.SharePoint;

namespace RH.PictureLibraryExif
{
    /// <summary>
    /// thx to Jeremy Sublett
    /// http://jeremysublett.com/archive/2008/05/30/getting-date-information-from-photos-in-sharepoint.aspx
    /// </summary>
    public class PictureLibraryExif : SPItemEventReceiver
    {
        public override void ItemAdded(SPItemEventProperties properties)
        {
            try
            {
                SPListItem item = properties.ListItem;

                if (Path.GetExtension(item.File.Name).ToLower() == ".jpg" || Path.GetExtension(item.File.Name).ToLower() == ".jpeg")
                {
                    Common.WriteULSVerbose(string.Format("found a jpeg. (Filename={0})", item.File.Name));
                    Stream stream = properties.ListItem.File.OpenBinaryStream();
                    Image image = Image.FromStream(stream);
                    DateTime? imageDate = GetImageDate(image);
                    if (imageDate != null)
                    {
                        Common.WriteULSVerbose("setting ImageCreateDate");
                        properties.ListItem[SPBuiltInFieldId.ImageCreateDate] = imageDate;// ((DateTime)imageDate).ToString("MM/dd/yyyy hh:mm tt");
                        item.SystemUpdate();
                    }
                }
            }
            catch (Exception ex)
            {
                Common.WriteULSError(string.Format("Error obtaining image datestamp: {0}", ex));
            }
        }

        private static DateTime? GetImageDate(Image image)
        {
            Common.WriteULSVerbose("Trying to find a DateTime from the exif data");
            int tagIndex = -1;

            DateTime? date = null;

            // find the index of the id we're looking for
            // Note: the Exif specification at http://exif.org specifies 0x9003 for
            // the date/time the image was taken
            for (int i = 0; i < image.PropertyIdList.Length; i++)
            {
                if (image.PropertyIdList[i] == Int32.Parse("9003", NumberStyles.HexNumber))
                {
                    tagIndex = i;
                    break;
                }
            }

            // return if the tag is not found
            if (tagIndex < 0) return null;

            // parse the date string which is in the format: yyyy:mm:dd hh:mm:ss
            string dateString;

            try
            {
                PropertyItem item = image.PropertyItems[tagIndex];
                dateString = Encoding.ASCII.GetString(item.Value);
                Common.WriteULSVerbose(string.Format("found a datetime: {0}", dateString));

                string[] dParts = dateString.Split(new[] { ":", " " }, StringSplitOptions.RemoveEmptyEntries);
                int year = Convert.ToInt32(dParts[0].Trim());
                int month = Convert.ToInt32(dParts[1].Trim());
                int day = Convert.ToInt32(dParts[2].Trim());
                int hour = Convert.ToInt32(dParts[3].Trim());
                int minute = Convert.ToInt32(dParts[4].Trim());
                int second = Convert.ToInt32(dParts[5].Trim());

                date = new DateTime(year, month, day, hour, minute, second);

            }
            catch { }
            // in case of an exception, we'll just ignore it and not return a date

            return date;
        }

    }
}